/*
 * Copyright (C) 2004 Fernando Herrera <fherrera@onirica.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <config.h>
#include <string.h>
#include <math.h>
#include <gst/gst.h>
#include <gst/gsterror.h>
#include <gst/gconf/gconf.h>
#include <gst/play/play.h>
#include <glib/gi18n.h>
#include <libgnomevfs/gnome-vfs-uri.h>
#include <libgnomevfs/gnome-vfs-file-size.h>
#include <libgnomevfs/gnome-vfs-utils.h>
#include <nautilus-burn-drive.h>
#include <nautilus-burn-recorder.h>
#include <sys/statvfs.h>


typedef struct {
	gboolean    cancel;
	char       *error;
} Converter;


const char *get_converter_error (Converter *conv);
Converter * create_converter (void);
gboolean    convert_file (Converter *conv, const char *orig, const char *dest);
void        convert_file_cancel (Converter *conv);
static void error_cb (GstElement *gstelement, GstElement *arg1, GError *arg2, gchar *arg3, Converter *conv);

static gboolean inited = FALSE;

Converter *
create_converter (void)
{
	Converter *conv;

	if (!inited) {
		gst_init (NULL, NULL);
		inited = TRUE;
	}

	conv = g_new0 (typeof (Converter), 1);
	return conv;
}

static void
error_cb (GstElement *gstelement,
          GstElement *arg1,
          GError *error,
          gchar *str,
	  Converter *conv)
{
	conv->error = str;
}

static GstElement *
create_pipeline (Converter *conv, const char *orig, const char *dest)
{
	GstElement *pipeline;
	GstElement *source;
	GstElement *decoder;
	GstElement *converter;
	GstElement *encoder;
	GstElement *sink;

	pipeline = gst_pipeline_new ("pipeline");

	source = gst_element_factory_make ("gnomevfssrc", "source");
	if (!source) {
		conv->error = g_strdup ("Could not create gnomevfssrc element");
		return NULL;
	}

	decoder = gst_element_factory_make ("spider", "spider");
	if (!decoder) {
		conv->error = g_strdup ("Could not create spider element");
		return NULL;
	}

	converter = gst_element_factory_make ("audioconvert", "converter");
	encoder = gst_element_factory_make ("wavenc", "encoder");
	sink = gst_element_factory_make ("filesink", "sink");

	gst_bin_add_many (GST_BIN (pipeline), source, decoder,
			  converter, encoder, sink, NULL);
	gst_element_link_many (source, decoder, converter,
			       encoder, sink, NULL);
	g_signal_connect (pipeline, "error", G_CALLBACK (error_cb), conv);

	g_object_set (G_OBJECT (source), "location", orig, NULL);
	g_object_set (G_OBJECT (sink), "location", dest, NULL);

	return pipeline;
}


gboolean
convert_file (Converter *conv, const char *orig, const char *dest)
{
	GstElement *pipeline;
	
	g_free (conv->error);
	conv->cancel = FALSE;

	pipeline = create_pipeline (conv, orig, dest);

	if (pipeline == NULL)
		return FALSE;
	
	gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_PLAYING);

	while (gst_bin_iterate (GST_BIN (pipeline))) {
		if (conv->cancel == TRUE ||
		    conv->error != NULL) {
			break;
		}
	}

	gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_NULL);
	g_object_unref (G_OBJECT (pipeline));

	return conv->error == NULL;
}

void
convert_file_cancel (Converter *conv)
{
	conv->cancel = TRUE;
}

const char *
get_converter_error (Converter *conv)
{
	return conv->error;
}

